import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AddressesServiceProxy, AddressDto, GetAddressForViewDto, SetOverridableAddressTypeInput  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditAddressModalComponent } from './create-or-edit-address-modal.component';
import { ViewAddressModalComponent } from './view-address-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { HttpClient } from '@angular/common/http';
import { AppConsts } from '@shared/AppConsts';
import { finalize } from 'rxjs/operators';
import { FileUpload } from 'primeng/fileupload';
import { formatDate } from '@angular/common';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import { SetOverridableAddressTypeModalComponent } from "./set-overridable-address-type-modal.component";

@Component({
    templateUrl: './addresses.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./addresses.component.less']
})
export class AddressesComponent extends AppComponentBase {

    @ViewChild('createOrEditAddressModal', { static: true }) createOrEditAddressModal: CreateOrEditAddressModalComponent;
    @ViewChild('viewAddressModalComponent', { static: true }) viewAddressModal: ViewAddressModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('ExcelFileUpload', { static: true }) excelFileUpload: FileUpload;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    
    @ViewChild('setOverridableAddressTypeModalComponent', { static: true }) overridableAddressTypeModal: SetOverridableAddressTypeModalComponent;

    overridableInput: SetOverridableAddressTypeInput = new SetOverridableAddressTypeInput();

    _entityTypeFullName = 'SprintTek.Shipping.Contacts.Address';
    entityHistoryEnabled = false;
    selectAll: boolean = false;
    uploadUrl: string;
    advancedFiltersAreShown = false;
    filterText = '';
    addressLine1Filter = '';
    addressLine2Filter = '';
    postalCodeValueFilter = '';
    countryFilter = '';
    stateFilter = '';
    cityFilter = '';
    geocodeLatitudeFilter = '';
    geocodeLongitudeFilter = '';
    addressTypeNameFilter = '';
    notValidateAddress = false;
    overridableAddressType = -1;
    tableRowIdsList = [] as any;
    tableRowNamesList = [] as any;
    addressIds = [] as any;

    constructor(
        injector: Injector,
        private _addressesServiceProxy: AddressesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _httpClient: HttpClient,
    ) {
        super(injector);
        this.uploadUrl = AppConsts.remoteServiceBaseUrl + '/Import/ImportFromExcel';
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

  
    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetAddressForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.address.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Address - " +role.address.addressLine1
        });
    }


    showOverridableAddressTypeModal(): void {

        if(this.tableRowIdsList.length != 0)
        {
            this.overridableAddressTypeModal.show();

        } else {
            this.notify.error(this.l('Please check atleast one in checkbox'));
        }
    }


    enableOverridableAddressType(id: number) : void {

        this.overridableInput.addressIds = [id];
        this.overridableInput.value = true;

        this._addressesServiceProxy.setOverridableAddressType(this.overridableInput)
            .subscribe(result => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.getAddresses();
            });
    }

    disableOverridableAddressType(id: number): void {
        this.overridableInput.addressIds = [id];
        this.overridableInput.value = false;

        this._addressesServiceProxy.setOverridableAddressType(this.overridableInput)
            .subscribe(result => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.getAddresses();
            });
    }


    getAddresses(event?: LazyLoadEvent) {

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._addressesServiceProxy.getAll(
            this.filterText,
            this.addressLine1Filter,
            this.addressLine2Filter,
            this.postalCodeValueFilter,
            this.countryFilter,
            this.stateFilter,
            this.cityFilter,
            this.geocodeLatitudeFilter,
            this.geocodeLongitudeFilter,
            this.addressTypeNameFilter,
            this.notValidateAddress,
            this.overridableAddressType,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;

            this.primengTableHelper.records.forEach(val => { val.selectedAddress = false });
            this.tableRowIdsList = [];
            this.tableRowNamesList = [];
            
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createAddress(): void {
        this.createOrEditAddressModal.show();
    }

    deleteAddress(address: AddressDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._addressesServiceProxy.delete(address.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    CheckAllOptions() {
        if (!this.selectAll) {
            this.primengTableHelper.records.forEach(val => { val.selectedAddress = false });
            this.tableRowIdsList = [];
            this.tableRowNamesList = [];
        } 
        else {
            this.primengTableHelper.records.forEach(val => { 
                this.tableRowIdsList.push(val.address.id);
                val.selectedAddress = true;
            });
            this.primengTableHelper.records.forEach(val => { 
                this.tableRowNamesList.push(val.addressTypeName);
            });
        } 
    }

    selectAddress(id: number, name: string) {
        if (this.tableRowIdsList.indexOf(id) == -1) {

            this.tableRowIdsList.push(id);
            this.tableRowNamesList.push(name);

        } else {
            this.tableRowIdsList.splice(this.tableRowIdsList.indexOf(id), 1);
            this.tableRowNamesList.splice(this.tableRowNamesList.indexOf(name), 1);
        }
    }
    

    validateAddress(address: AddressDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._addressesServiceProxy.validateAddress(address)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('AddressValidated'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._addressesServiceProxy.getAddressesToExcel(
        this.filterText,
            this.addressLine1Filter,
            this.addressLine2Filter,
            this.postalCodeValueFilter,
            this.geocodeLatitudeFilter,
            this.addressTypeNameFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }

    
    uploadExcel(data: { files: File }): void {
        const formData: FormData = new FormData();
        const file = data.files[0];
        formData.append('file', file, file.name);
        formData.append('key','addresses');
        // formData.append('entity','addresses','addresses');

        this._httpClient
            .post<any>(this.uploadUrl, formData)
            .pipe(finalize(() => this.excelFileUpload.clear()))
            .subscribe(response => {
                if (response.success) {
                    this.notify.success(this.l('ImportAddressProcessStart'));
                } else if (response.error != null) {
                    this.notify.error(this.l('ImportAddressUploadFailed'));
                }
            });
    }

    onUploadExcelError(): void {
        this.notify.error(this.l('ImportAddressUploadFailed'));
    }
}
