import { Component, EventEmitter, Injector, OnInit, Output, ViewChild } from '@angular/core';
import { AddMemberModalComponent } from '@app/admin/organization-units/add-member-modal.component';
import { AppComponentBase } from '@shared/common/app-component-base';
import { OrganizationUnitServiceProxy, OrganizationUnitUserListDto, FleetRateServiceProxy, FleetRateListDto, FleetUserListDto } from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { IUserWithOrganizationUnit } from '@app/admin/organization-units/user-with-organization-unit';
import { IUsersWithOrganizationUnit } from '@app/admin/organization-units/users-with-organization-unit';
import { IBasicOrganizationUnitInfo } from '@app/admin/organization-units/basic-organization-unit-info';
import { FleetAddRateComponent } from './fleet-add-rate.component';
import { IFleetUnitInfo } from './fleets-unit-info';
import { IRatesWithFleet } from './rates-with-fleet.component';
import { IRateWithFleet } from './rate-with-fleet.component';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';

@Component({
    selector: 'fleet-rates',
    templateUrl: './fleet-rates.component.html'
})
export class FleetRatesComponent extends AppComponentBase implements OnInit {

    @Output() rateRemoved = new EventEmitter<IRateWithFleet>();
    @Output() rateAdded = new EventEmitter<IRatesWithFleet>();

    @ViewChild('fleetAddRateComponent', {static: true}) fleetAddRateComponent: FleetAddRateComponent;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Fleets.FleetRate';
    entityHistoryEnabled = false;

    private _organizationUnit: IFleetUnitInfo = null;

    constructor(
        injector: Injector,
        private _organizationUnitService: OrganizationUnitServiceProxy,
        private _fleetRateService: FleetRateServiceProxy
    ) {
        super(injector);
    }

    get organizationUnit(): IFleetUnitInfo {
        return this._organizationUnit;
    }

    set organizationUnit(ou: IFleetUnitInfo) {
        if (this._organizationUnit === ou) {
            return;
        }

        this._organizationUnit = ou;
        this.fleetAddRateComponent.fleetId = ou.id;
        if (ou) {
            this.refreshMembers();
        }

        if (ou) {
            this.refreshMembers();
        }
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: FleetUserListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Fleet Rate - " +role.user.name +" " +role.user.surname
        });
    }

    getFleetRates(event?: LazyLoadEvent) {
        
        if (!this._organizationUnit) {
            return;
        }

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();
        this._fleetRateService.getFleetRate(
            undefined,
            this._organizationUnit.id,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            //result.items.rate.locationName = this.getLocationName(result.items.rate.locationId);
            this.primengTableHelper.records = result.items;
            
            this.spinnerService.hide();
            
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    refreshMembers(): void {
        this.reloadPage();
    }

    openAddMemberModal(): void {
        this.fleetAddRateComponent.show(this._organizationUnit.id);
    }

    removeRate(rate: FleetRateListDto): void {
        this.message.confirm('',
            this.l('AreYouSure'),
            isConfirmed => {
                if (isConfirmed) {
                    this._fleetRateService
                        .deleteFleetRate(rate.id)
                        .subscribe(() => {
                            this.notify.success(this.l('SuccessfullyRemoved'));
                            this.rateRemoved.emit({
                                rateId: rate.rateId,
                                ouId: this.organizationUnit.id
                            });
                            this.getFleetRates();
                        });
                }
            }
        );
    }

    addRate(data: any): void {
        this.rateAdded.emit({
            rateIds: data.rateIds,
            ouId: data.ouId
        });

        this.refreshMembers();
    }

}
