import { Component, EventEmitter, Injector, OnInit, Output, ViewChild } from '@angular/core';
import { AddMemberModalComponent } from '@app/admin/organization-units/add-member-modal.component';
import { AppComponentBase } from '@shared/common/app-component-base';
import { OrganizationUnitServiceProxy, OrganizationUnitUserListDto, FleetUserServiceProxy, FleetUserListDto } from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { IUserWithOrganizationUnit } from '@app/admin/organization-units/user-with-organization-unit';
import { IUsersWithOrganizationUnit } from '@app/admin/organization-units/users-with-organization-unit';
import { IBasicOrganizationUnitInfo } from '@app/admin/organization-units/basic-organization-unit-info';
import { FleetAddDriverComponent } from './fleet-add-driver.component';
import { IFleetUnitInfo } from './fleets-unit-info';
import { IDriversWithFleet } from './drivers-with-fleet.component';
import { IDriverWithFleet } from './driver-with-fleet.component';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';

@Component({
    selector: 'fleet-users',
    templateUrl: './fleet-users.component.html'
})
export class FleetUsersComponent extends AppComponentBase implements OnInit {

    @Output() driverRemoved = new EventEmitter<IDriverWithFleet>();
    @Output() driverAdded = new EventEmitter<IDriversWithFleet>();

    @ViewChild('fleetAddDriverComponent', {static: true}) fleetAddDriverComponent: FleetAddDriverComponent;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Fleets.FleetUser';
    entityHistoryEnabled = false;

    private _organizationUnit: IFleetUnitInfo = null;

    constructor(
        injector: Injector,
        private _organizationUnitService: OrganizationUnitServiceProxy,
        private _fleetUserService: FleetUserServiceProxy
    ) {
        super(injector);
    }

    get organizationUnit(): IFleetUnitInfo {
        return this._organizationUnit;
    }

    set organizationUnit(ou: IFleetUnitInfo) {
        if (this._organizationUnit === ou) {
            return;
        }

        this._organizationUnit = ou;
        this.fleetAddDriverComponent.fleetId = ou.id;
        if (ou) {
            this.refreshMembers();
        }
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: FleetUserListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Fleet User - " +role.user.name +" " +role.user.surname
        });
    }

    getFleetDrivers(event?: LazyLoadEvent) {
        if (!this._organizationUnit) {
            return;
        }

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();
        this._fleetUserService.getFleetUserPaged(
            undefined,
            undefined,
            this._organizationUnit.id,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    refreshMembers(): void {
        this.reloadPage();
    }

    openAddMemberModal(): void {
        this.fleetAddDriverComponent.show();
    }

    removeDriver(driver: FleetUserListDto): void {
        this.message.confirm('',
            this.l('AreYouSure'),
            isConfirmed => {
                if (isConfirmed) {
                    this._fleetUserService
                        .deleteFleetUser(driver.id)
                        .subscribe(() => {
                            this.notify.success(this.l('SuccessfullyRemoved'));
                            this.driverRemoved.emit({
                                userId: driver.userId,
                                ouId: this.organizationUnit.id
                            });
                            this.getFleetDrivers();
                        });
                }
            }
        );
    }

    addDriver(data: any): void {
        this.driverAdded.emit({
            userIds: data.userIds,
            ouId: data.ouId
        });

        this.refreshMembers();
    }
}
