import { ChangeDetectorRef, Component, EventEmitter, Injector, OnInit, Output, ViewChild } from '@angular/core';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { OrganizationUnitServiceProxy, OrganizationUnitRoleListDto, FleetVehicleServiceProxy, PagedResultDtoOfFleetVehicleListDto, FleetVehicleListDto } from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { IRoleWithOrganizationUnit } from '@app/admin/organization-units/role-with-organization-unit';
import { IRolesWithOrganizationUnit } from '@app/admin/organization-units/roles-with-organization-unit';
import { IBasicOrganizationUnitInfo } from '@app/admin/organization-units/basic-organization-unit-info';
import { FleetAddVehicleComponent } from './fleet-add-vehicle.component';
import { IFleetUnitInfo } from './fleets-unit-info';
import { IVehicleWithFleet } from './vehicle-with-fleet.component';
import { IVehiclesWithFleet } from './vehicles-with-fleet.component';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';

@Component({
    selector: 'fleet-vehicles',
    templateUrl: './fleet-vehicles.component.html',
    animations: [appModuleAnimation()]
})
export class FleetVehiclesComponent extends AppComponentBase implements OnInit {

    @Output() vehicleRemoved = new EventEmitter<IVehicleWithFleet>();
    @Output() vehicleAdded = new EventEmitter<IVehiclesWithFleet>();

    @ViewChild('fleetAddVehicleComponent', {static: true}) fleetAddVehicleComponent: FleetAddVehicleComponent;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Fleets.FleetVehicle';
    entityHistoryEnabled = false;

    private _organizationUnit: IFleetUnitInfo = null;

    constructor(
        injector: Injector,
        private _changeDetector: ChangeDetectorRef,
        private _organizationUnitService: OrganizationUnitServiceProxy,
        private _fleetVehicleService: FleetVehicleServiceProxy
    ) {
        super(injector);
    }

    get organizationUnit(): IFleetUnitInfo {
        
        return this._organizationUnit;
    }

    set organizationUnit(ou: IFleetUnitInfo) {
        if (this._organizationUnit === ou) {
            return;
        }

        this._organizationUnit = ou;
        this.fleetAddVehicleComponent.fleetId = ou.id;
        if (ou) {
            this.refreshRoles();
        }
        console.log("set")
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: FleetVehicleListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Fleet Vehicle - " +role.vehicle.name
        });
    }

    getVehicles(event?: LazyLoadEvent) {
        if (!this._organizationUnit) {
            return;
        }

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }
        this.spinnerService.show();
        this._fleetVehicleService.getFleetVehiclePaged(
            undefined,
            this._organizationUnit.id,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    refreshRoles(): void {
        this.reloadPage();
    }

    openAddVehicleModal(): void {
        this.fleetAddVehicleComponent.show();
    }

    removeVehicle(vehicle: FleetVehicleListDto): void {
        this.message.confirm(
            '','Are you sure?',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._fleetVehicleService.deleteFleetVehicle(vehicle.id)
                        .subscribe(() => {
                            this.vehicleRemoved.emit({
                                vehicleId: vehicle.vehicleId,
                                ouId: this.organizationUnit.id
                            });
                            this.getVehicles();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    addVehicle(data: any): void {
        this.vehicleAdded.emit({
            vehicleIds: data.vehicleIds,
            ouId: data.ouId
        });

        this.refreshRoles();
    }
}
